@@ -1,39 +1,35 @@
+use std::io;
 use std::os::windows::prelude::*;
 use std::process::Child;
-use std::io;
+use std::time::Duration;
 
 use kernel32::*;
 use winapi::*;
 
-const STILL_ACTIVE: DWORD = 259;
-
 #[derive(Eq, PartialEq, Copy, Clone, Debug)]
 pub struct ExitStatus(DWORD);
 
-pub fn wait_timeout_ms(child: &mut Child, ms: u32)
+pub fn wait_timeout(child: &mut Child, dur: Duration)
                        -> io::Result<Option<ExitStatus>> {
-    if let Some(status) = try!(try_wait(child)) {
-        return Ok(Some(status))
-    }
+    let ms = dur.as_secs().checked_mul(1000).and_then(|amt| {
+        amt.checked_add(dur.subsec_nanos() as u64)
+    }).expect("failed to convert duration to milliseconds");
+    let ms = if ms > (DWORD::max_value() as u64) {
+        DWORD::max_value()
+    } else {
+        ms as DWORD
+    };
     unsafe {
         match WaitForSingleObject(child.as_raw_handle(), ms) {
-            WAIT_OBJECT_0 |
-            WAIT_TIMEOUT => {}
+            WAIT_OBJECT_0 => {}
+            WAIT_TIMEOUT => return Ok(None),
             _ => return Err(io::Error::last_os_error()),
         }
-    }
-    try_wait(child)
-}
-
-fn try_wait(child: &mut Child) -> io::Result<Option<ExitStatus>> {
-    unsafe {
         let mut status = 0;
         if GetExitCodeProcess(child.as_raw_handle(), &mut status) == FALSE {
             Err(io::Error::last_os_error())
-        } else if status != STILL_ACTIVE {
-            Ok(Some(ExitStatus(status)))
         } else {
-            Ok(None)
+            Ok(Some(ExitStatus(status)))
         }
     }
 }
